@@ -25,50 +25,99 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <fcntl.h>
+
+#ifndef O_LARGEFILE
+#define O_LARGEFILE 0
+#endif
 
 #define NAME "muroarstream"
 
+static int str2chans(const char * chans) {
+ if ( !strcasecmp(chans, "stereo") )
+  return MUROAR_CM_STEREO;
+ if ( !strcasecmp(chans, "mono") )
+  return MUROAR_CM_MONO;
+ if ( !strcasecmp(chans, "default") )
+  return MUROAR_AID_CHANNELS;
+ return atoi(chans);
+}
+
+static int str2codec(const char * codec) {
+ if ( !strcasecmp(codec, "default") )
+  return -1;
+ if ( !strcasecmp(codec, "pcm") || !strcasecmp(codec, "pcm_s") )
+  return MUROAR_CODEC_PCM_S;
+ if ( !strcasecmp(codec, "pcm_u") )
+  return MUROAR_CODEC_PCM_U;
+ if ( !strcasecmp(codec, "alaw") )
+  return MUROAR_CODEC_ALAW;
+ if ( !strcasecmp(codec, "mulaw") || !strcasecmp(codec, "ulaw") )
+  return MUROAR_CODEC_MULAW;
+ if ( !strcasecmp(codec, "ogg_vorbis") || !strcasecmp(codec, "vorbis") )
+  return MUROAR_CODEC_OGG_VORBIS;
+ if ( !strcasecmp(codec, "dmx512") )
+  return MUROAR_CODEC_DMX512;
+ if ( !strcasecmp(codec, "roardmx") )
+  return MUROAR_CODEC_ROARDMX;
+ if ( !strcasecmp(codec, "midi") )
+  return MUROAR_CODEC_MIDI;
+ return atoi(codec);
+}
+
 void usage (char * progname) {
  printf("Usage: %s [OPTIONS]\n", progname);
  printf("\n");
  printf("Options:\n");
  printf("    --help              - Display this help.\n"
+        " -f --file     FILE     - File to read from or write to. (default: stdin/stdout)\n"
         "    --server   SERVER   - Sets the server. (default: autodetect)\n"
         " -R --rate     RATE     - Sets the sample rate (default: 44100Hz)\n"
         " -C --channels CHANS    - Sets the number of channels (default: 2 channels)\n"
         " -B --bits     BITS     - Sets the number of bits per sample (default: 16bit)\n"
+        " -E --codec    CODEC    - Set codec to use (default: depend on used subsystem)\n"
+        "    --play              - Use Playback mode (send data to the server) (default)\n"
+        "    --monitor           - Use monitor mode (get a copy of the played data from the server)\n"
         "    --wave              - Use Waveform subsystem (default)\n"
         "    --midi              - Use MIDI subsystem\n"
         "    --light             - Use Light Control subsystem\n"
        );
 }
 
-void run_stream(int stream, int tx) {
+void run_stream(int stream, int file, int tx) {
  char buf[1024];
  ssize_t len;
 
  if ( tx ) {
-  while ((len = read(STDIN_FILENO, buf, sizeof(buf))) > 0)
+  if ( file == -1 )
+   file = STDIN_FILENO;
+
+  while ((len = read(file, buf, sizeof(buf))) > 0)
    if ( muroar_write(stream, buf, len) != len )
     return;
  } else {
+  if ( file == -1 )
+   file = STDOUT_FILENO;
+
   while ((len = muroar_read(stream, buf, sizeof(buf))) > 0)
-   if ( write(STDOUT_FILENO, buf, len) != len )
+   if ( write(file, buf, len) != len )
     return;
  }
 }
 
 int main (int argc, char *argv[]) {
  int fh;
- int dir   = MUROAR_PLAY_WAVE;
- int codec = -1;     /* use default depending in dir */
- int rate  =  MUROAR_AID_RATE;
- int chans =  MUROAR_AID_CHANNELS;
- int bits  =  MUROAR_AID_BITS;
- int tx    =  1;
+ int filefh = -1;
+ int dir    = MUROAR_PLAY_WAVE;
+ int codec  = -1;     /* use default depending in dir */
+ int rate   =  MUROAR_AID_RATE;
+ int chans  =  MUROAR_AID_CHANNELS;
+ int bits   =  MUROAR_AID_BITS;
+ int tx     =  1;
  int i;
  char * k;
  char * server = NULL;
+ const char * file   = NULL;
 
  for (i = 1; i < argc; i++) {
   k = argv[i];
@@ -76,14 +125,18 @@ int main (int argc, char *argv[]) {
   if ( !strcmp(k, "--help") ) {
    usage(argv[0]);
    return 0;
+  } else if ( !strcmp(k, "--file") || !strcmp(k, "-f") ) {
+   file = argv[++i];
   } else if ( !strcmp(k, "--server") ) {
    server = argv[++i];
   } else if ( !strcmp(k, "--rate") || !strcmp(k, "-R") ) {
    rate = atoi(argv[++i]);
   } else if ( !strcmp(k, "--channels") || !strcmp(k, "-C") ) {
-   chans = atoi(argv[++i]);
+   chans = str2chans(argv[++i]);
   } else if ( !strcmp(k, "--bits") || !strcmp(k, "-B") ) {
    bits = atoi(argv[++i]);
+  } else if ( !strcmp(k, "--codec") || !strcmp(k, "-E") ) {
+   codec = str2codec(argv[++i]);
   } else if ( !strcmp(k, "--play") ) {
    tx = 1;
   } else if ( !strcmp(k, "--monitor") ) {
@@ -118,6 +171,14 @@ int main (int argc, char *argv[]) {
   }
  }
 
+ if ( file != NULL ) {
+  filefh = open(file, O_LARGEFILE|(tx ? O_RDONLY : O_WRONLY|O_CREAT|O_TRUNC), 0666);
+  if ( filefh == -1 ) {
+   fprintf(stderr, "Error: Can not open file: %s\n", file);
+   return 5;
+  }
+ }
+
  // Connect to server:
  if ( (fh = muroar_connect(server, NAME)) == -1 ) {
   fprintf(stderr, "Error: Can not connect to server.\n");
@@ -130,10 +191,13 @@ int main (int argc, char *argv[]) {
   return 2;
  }
 
- run_stream(fh, tx);
+ run_stream(fh, filefh, tx);
 
  muroar_close(fh);
 
+ if ( filefh != -1 )
+  close(filefh);
+
  return 0;
 }
 
